﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;
using System;
using System.Collections.Generic;

public class ActionTrigger : MonoBehaviour 
{
	public string textToDisplay;
	public string textForFamilyIfProtected = "Thanks, we should make it through the night now.";
	CanvasGroupFader _gameTextFader;
	Text _gameTextTxt; 

	public bool familyDiscovered = false;

	public string itemName;
	public CanvasGroupFader popUp;
	CanvasGroupFader pressSpace;

	MapControl mapControl;

	public Game.PICKUPS pickup;

	bool waitingForKeyPress = false;

	bool continueAfterAction = false;

	public enum ACTION_TYPE
	{
		NONE = 0,
		STRENGTHEN = 1,
		TIME_WASTE = 2,
		MAINTENANCE = 3,
		SLEEP = 4,
		EXPLORE = 5,
		EXIT = 6,
		PICKUP = 7,
		POP_UP = 8,
		FAMILY = 9
	}

	public ACTION_TYPE actionType;

	public Strengthening strengthening;

	int currentResourceCost = 1;

	public bool broken = false;
	bool _interacting = false;
	bool _interactable = false;
	bool _available = true;

	string dayOneText = "I Don't have time for that, I need to fortify this place and service my appliances!";
	string cantSleepText = "I Probably shouldn't sleep yet. Midnight is more like my bedtime.";
	string sleepText = "I could really do with sleeping now.";
	string hasKeyText = "I could try to find other survivors.";
	string noKey = "I can't remember where I put the key for that door";
	string noKeyUnderPot = "I think the key is under the plant pot over there.";
	string brokenText = "This is destroyed, it can never be fixed";
	string outOfResource = "I don't have enough resources to do this";

	#region STRENGTHEN

	public enum STRENGTHEN_TIERS
	{
		NONE = 0,
		FIRST_TIER = 1,
		SECOND_TIER = 2,
		THIRD_TIER = 3,
		REINFORCE = 4,
		BROKEN = 5
	}

	Dictionary<STRENGTHEN_TIERS, string> textForStrengthenTiers = new Dictionary<STRENGTHEN_TIERS, string>()
	{
		{STRENGTHEN_TIERS.NONE, "Strengthen To 50% (Consumes 1 Resource & 1 Hour)"},
		{STRENGTHEN_TIERS.FIRST_TIER, "Strengthen To 75% (Consumes 1 Resource & 1 Hour)"},
		{STRENGTHEN_TIERS.SECOND_TIER, "Strengthen To 100% (Consumes 1 Resource & 1 Hour)"},
		{STRENGTHEN_TIERS.THIRD_TIER, "Reinforce (125%) (Consumes 1 Resource & 1 Hour)"},
		{STRENGTHEN_TIERS.REINFORCE, ""},
		{STRENGTHEN_TIERS.BROKEN, "This can never be repaired"}
	};

	public STRENGTHEN_TIERS _currentStrengthenTier = STRENGTHEN_TIERS.NONE;
	STRENGTHEN_TIERS currentStrengthenTier
	{
		set 
		{
			_currentStrengthenTier = value;
			textToDisplay = textForStrengthenTiers [_currentStrengthenTier];
			_gameTextTxt.text = textToDisplay;
		}
	}

	#endregion

	#region MAINTENANCE

	enum MAINTENANCE_TIERS
	{
		WORKING = 0,
		SERVICED = 1,
		BUSTED = 2,
		BROKEN = 3
	}

	Dictionary<MAINTENANCE_TIERS, string> textForMaintenanceTiers = new Dictionary<MAINTENANCE_TIERS, string>()
	{
		{MAINTENANCE_TIERS.WORKING, "Service (Consumes 1 Resource & 1 Hour)"},
		{MAINTENANCE_TIERS.SERVICED, ""},
		{MAINTENANCE_TIERS.BUSTED, "Repair & Service (Consumes 2 Resource Points & 2 Hours)"},
		{MAINTENANCE_TIERS.BROKEN, "It's busted, i'm screwed!"}
	};


	MAINTENANCE_TIERS _currentMaintenanceTier = MAINTENANCE_TIERS.WORKING;
	MAINTENANCE_TIERS currentMaintenanceTier
	{
		set 
		{
			_currentMaintenanceTier = value;
			textToDisplay = textForMaintenanceTiers [_currentMaintenanceTier];
			_gameTextTxt.text = textToDisplay;

		}
	}

	#endregion

	public enum FAMILY_TIERS
	{
		VULNERABLE = 0,
		PROTECTED = 1
	}

	public FAMILY_TIERS _currentFamilyTier = FAMILY_TIERS.VULNERABLE;

	void Awake()
	{
		_gameTextFader = GameObject.Find ("GameText").GetComponent<CanvasGroupFader> ();
		_gameTextTxt = _gameTextFader.GetComponent<Text> ();

		pressSpace = GameObject.Find ("PressSpace").GetComponent<CanvasGroupFader> ();
		mapControl = GameObject.Find ("MapBG").GetComponent<MapControl> ();

		if (actionType == ACTION_TYPE.FAMILY)
		{
			currentResourceCost = UnityEngine.Random.Range (1, 3);
			textToDisplay = textToDisplay + " (Consumes " + currentResourceCost.ToString() + 
				" Resource & " + currentResourceCost.ToString() + " Hour(s))";
		}
	}

	void OnTriggerEnter2D(Collider2D other)
	{
		Game.Instance.UpdateAndShowItemName (itemName);
		if ((actionType == ACTION_TYPE.MAINTENANCE && _currentMaintenanceTier != MAINTENANCE_TIERS.SERVICED) 
			|| (actionType == ACTION_TYPE.STRENGTHEN && _currentStrengthenTier != STRENGTHEN_TIERS.REINFORCE) 
			|| (actionType == ACTION_TYPE.FAMILY && _currentFamilyTier != FAMILY_TIERS.PROTECTED))
		{
			if ((Game.Instance.currentNumResources - currentResourceCost) >= 0)
			{
				_gameTextTxt.text = textToDisplay;
				_gameTextFader.FadeIn (0.3f);
				_interactable = true;
			}
			else
			{
				_gameTextTxt.text = outOfResource;
				_gameTextFader.FadeIn (0.3f);
			}
		}
		else
		{
			_gameTextTxt.text = textToDisplay;
			_gameTextFader.FadeIn (0.3f);
			_interactable = true;
		}

		if (actionType == ACTION_TYPE.FAMILY)
		{
			Game.Instance.humanContactForTheDay = true;
			familyDiscovered = true;
			GameObject.Find ("MapBG").GetComponent<MapControl> ().apartmentLookups.Find (x => x.physicalApartmentName == Player.Instance.currentApartment.gameObject.name).mapApartment.apartmentDetail.owners = itemName;
			GameObject.Find ("MapBG").GetComponent<MapControl> ().apartmentLookups.Find (x => x.physicalApartmentName == Player.Instance.currentApartment.gameObject.name).mapApartment.containsDiscoveredFamily = true;
		}

	}

	void Update()
	{
		if (actionType == ACTION_TYPE.MAINTENANCE)
		{
			if (_currentMaintenanceTier == MAINTENANCE_TIERS.BUSTED)
			{
				currentResourceCost = 2;
			}
			else
			{
				currentResourceCost = 1;
			}
		}

		if (broken)
		{
			_gameTextTxt.text = brokenText;
			return;
		}

		if (!_interactable)
			return;

		switch (actionType)
		{
		case ACTION_TYPE.MAINTENANCE:
			if (_currentMaintenanceTier == MAINTENANCE_TIERS.BUSTED)
			{
				currentResourceCost = 2;
			}
			else
			{
				currentResourceCost = 1;
			}
			textToDisplay = textForMaintenanceTiers [_currentMaintenanceTier];
			_gameTextTxt.text = textToDisplay;
			break;

		case ACTION_TYPE.STRENGTHEN:
			textToDisplay = textForStrengthenTiers [_currentStrengthenTier];
			_gameTextTxt.text = textToDisplay;
			break;

		case ACTION_TYPE.TIME_WASTE:
			if (Game.Instance.isDayOne && Game.Instance.hoursUsed < 15)
			{
				_gameTextTxt.text = dayOneText;
				_available = false;
			}
			else
			{
				Debug.Log (textToDisplay);
				_gameTextTxt.text = textToDisplay;
				_available = true;
			}
			break;
		case ACTION_TYPE.SLEEP:
			if (Game.Instance.hoursUsed < 16)
			{
				_gameTextTxt.text = cantSleepText;
				_available = false;
			}
			else
			{
				_gameTextTxt.text = sleepText;
				_available = true;
			}
			break;

		case ACTION_TYPE.EXPLORE:
			if (Game.Instance.hasKey)
			{
				_gameTextTxt.text = hasKeyText;
				_available = true;
			}
			else
			{
				_gameTextTxt.text = Game.Instance.currentDay >= 2 ? noKeyUnderPot : noKey;
				_available = false;
			}
			break;

		case ACTION_TYPE.FAMILY:
			if (_currentFamilyTier == FAMILY_TIERS.VULNERABLE)
			{
				_gameTextTxt.text = textToDisplay;
				_available = true;
			}
			else
			{
				_gameTextTxt.text = textForFamilyIfProtected;
				_available = false;
			}
			break;
		}
		
		// can't
		if (actionType == ACTION_TYPE.MAINTENANCE)
		{
			if (_currentMaintenanceTier == MAINTENANCE_TIERS.SERVICED)
			{
				return;
			}
		}

		if (actionType == ACTION_TYPE.STRENGTHEN)
		{
			if (_currentStrengthenTier == STRENGTHEN_TIERS.REINFORCE)
			{
				return;
			}
		}

		if (actionType == ACTION_TYPE.FAMILY)
		{
			if (_currentFamilyTier == FAMILY_TIERS.PROTECTED)
			{
				return;
			}
		}
			
		if (!_interacting && _interactable && _available)
		{
			if (Input.GetKeyDown (KeyCode.Space))
			{
				_interacting = true;
				StartCoroutine (DoInteraction ());
			}
		}

		if (waitingForKeyPress)
		{
			if (Input.GetKeyDown (KeyCode.Space))
			{
				continueAfterAction = true;
				pressSpace.FadeOut (0.1f);
			}
		}
	}

	void OnTriggerExit2D(Collider2D other)
	{
		_gameTextTxt.text = "";
		_gameTextFader.FadeOut (0.3f);
		_interactable = false;
		Game.Instance.HideItemName ();
	}

	IEnumerator DoInteraction()
	{
		_gameTextFader.FadeOut (0.2f);
		Debug.Log ("Doing Interaction");
		Player.Instance.interacting = true;

		yield return new WaitForSeconds (0.5f);
		DecideInteraction ();

		while(!continueAfterAction)
		{
			yield return null;
		}

		if (actionType == ACTION_TYPE.POP_UP)
		{
			if (popUp != null)
			{
				popUp.FadeOut (0.1f);
			}
		}
		continueAfterAction = false;
		waitingForKeyPress = false;
		_gameTextFader.FadeIn(0.1f);

		yield return new WaitForSeconds (0.2f);

		Player.Instance.interacting = false;
		_interacting = false;
	}

	void DecideInteraction()
	{
		switch (actionType)
		{
		case ACTION_TYPE.MAINTENANCE:
			if (_currentMaintenanceTier == MAINTENANCE_TIERS.BUSTED)
			{
				Game.Instance.SpendTime (this, 2);
				Game.Instance.SpendResource (this, 2);
			}
			else
			{
				Game.Instance.SpendTime (this);
				Game.Instance.SpendResource (this);
			}
			currentMaintenanceTier = GetNextMaintenanceTier (_currentMaintenanceTier);
			continueAfterAction = true;
			break;

		case ACTION_TYPE.STRENGTHEN:
			Game.Instance.SpendTime (this);
			Game.Instance.SpendResource (this);
			currentStrengthenTier = GetNextStrengthenTier (_currentStrengthenTier);
			strengthening.Set (_currentStrengthenTier);
			continueAfterAction = true;
			break;

		case ACTION_TYPE.TIME_WASTE:
			Game.Instance.SpendTime (this);
			if (!Game.Instance.isDayOne)
			{
				Game.Instance.AddToTimewasters(this);
			}
			continueAfterAction = true;
			break;

		case ACTION_TYPE.SLEEP:
			ResetStuffForAdvance ();
			Game.Instance.AdvanceDay ();
			continueAfterAction = true;
			break;

		case ACTION_TYPE.PICKUP:
			GetComponent<Collider2D> ().enabled = false;
			Game.Instance.Pickup (pickup);
			_gameTextTxt.text = "";
			_available = false;
			continueAfterAction = true;
			break;

		case ACTION_TYPE.POP_UP:
			popUp.FadeIn (0.5f);
			waitingForKeyPress = true;
			pressSpace.FadeIn (0.1f);
			break;

		case ACTION_TYPE.EXPLORE:
			ResetStuffForAdvance ();
			mapControl.currentActionTrigger = this;
			mapControl.enabled = true;
			break;

		case ACTION_TYPE.FAMILY:
			Game.Instance.SpendTime (this);
			Game.Instance.SpendResource (this, currentResourceCost);
			_currentFamilyTier = FAMILY_TIERS.PROTECTED;
			continueAfterAction = true;
			break;
		}
	}

	public void ResetStuffForAdvance()
	{
		Player.Instance.transitioning = true;
		Game.Instance.HideItemName ();
		_gameTextTxt.text = "";
		_interacting = false;
		_interactable = false;
		_gameTextFader.FadeOut (0.3f);
		Player.Instance.interacting = false;
	}

	MAINTENANCE_TIERS GetPreviousMaintenanceTier(MAINTENANCE_TIERS mt)
	{
		switch (mt)
		{
		case MAINTENANCE_TIERS.BUSTED:
			broken = true;
			return MAINTENANCE_TIERS.BROKEN;

			// max - no more to go to
		case MAINTENANCE_TIERS.SERVICED:
			return MAINTENANCE_TIERS.WORKING;

		case MAINTENANCE_TIERS.WORKING:
			return MAINTENANCE_TIERS.BUSTED;
		}

		return mt;
	}

	MAINTENANCE_TIERS GetNextMaintenanceTier(MAINTENANCE_TIERS mt)
	{
		switch (mt)
		{
		case MAINTENANCE_TIERS.BUSTED:
			return MAINTENANCE_TIERS.SERVICED;

			// max - no more to go to
		case MAINTENANCE_TIERS.SERVICED:
			return MAINTENANCE_TIERS.SERVICED;

		case MAINTENANCE_TIERS.WORKING:
			return MAINTENANCE_TIERS.SERVICED;
		}

		return mt;
	}


	STRENGTHEN_TIERS GetNextStrengthenTier(STRENGTHEN_TIERS st)
	{
		switch (st)
		{
		case STRENGTHEN_TIERS.NONE:
			return STRENGTHEN_TIERS.FIRST_TIER;

			// max - no more to go to
		case STRENGTHEN_TIERS.FIRST_TIER:
			return STRENGTHEN_TIERS.SECOND_TIER;

		case STRENGTHEN_TIERS.SECOND_TIER:
			return STRENGTHEN_TIERS.THIRD_TIER;

		case STRENGTHEN_TIERS.THIRD_TIER:
			return STRENGTHEN_TIERS.REINFORCE;

		case STRENGTHEN_TIERS.REINFORCE:
			return STRENGTHEN_TIERS.REINFORCE;
		}

		return st;
	}

	STRENGTHEN_TIERS GetPreviousStrengthenTier(STRENGTHEN_TIERS st)
	{
		switch (st)
		{
		case STRENGTHEN_TIERS.NONE:
			return STRENGTHEN_TIERS.FIRST_TIER;

			// max - no more to go to
		case STRENGTHEN_TIERS.FIRST_TIER:
			broken = true;
			return STRENGTHEN_TIERS.BROKEN;

		case STRENGTHEN_TIERS.SECOND_TIER:
			return STRENGTHEN_TIERS.FIRST_TIER;

		case STRENGTHEN_TIERS.THIRD_TIER:
			return STRENGTHEN_TIERS.SECOND_TIER;

		case STRENGTHEN_TIERS.REINFORCE:
			return STRENGTHEN_TIERS.THIRD_TIER;

		case STRENGTHEN_TIERS.BROKEN:
			return STRENGTHEN_TIERS.BROKEN;
		}

		return st;
	}

	public void ApplyDetriment(int multiplier)
	{
		switch (actionType)
		{
		case ACTION_TYPE.MAINTENANCE:
			currentMaintenanceTier = GetPreviousMaintenanceTier (_currentMaintenanceTier);
			break;

		case ACTION_TYPE.STRENGTHEN:
			currentStrengthenTier = GetPreviousStrengthenTier (_currentStrengthenTier);
			strengthening.Set (_currentStrengthenTier);
			break;

		case ACTION_TYPE.TIME_WASTE:
			// don't break
			break;

		case ACTION_TYPE.SLEEP:
			//don't break
			break;

		case ACTION_TYPE.FAMILY:
			_currentFamilyTier = FAMILY_TIERS.VULNERABLE;
			break;
		}
	}
}
