﻿using UnityEngine;
using System.Collections;

public class CameraScript : MonoBehaviour 
{
	// camera target
	Transform _target;

	bool _transitioning = false;
	public bool transitioning
	{
		set
		{
			_transitioning = value;
		}
	}

	// save the old pos for bounds checks
	Vector3 _oldPos;

	// room values
	public Room currentRoom;
	SpriteRenderer _currentRoomSprite;

	void Awake()
	{
		// find the player and set them as the target
		_target = GameObject.Find ("Player").transform;
		_currentRoomSprite = currentRoom.GetComponentInChildren<SpriteRenderer> ();

		// save pos
		Vector3 pos = transform.position;

		// adjust x
		pos.x = currentRoom.backgroundSprite.bounds.min.x + OrthographicBounds (Camera.main, transform.position).size.x * 0.5f;

		// apply!
		transform.position = pos;
	}
		
	void LateUpdate()
	{
		if (!_transitioning)
		{
			// get the old pos
			_oldPos = transform.position;

			// update the new pos
			Vector3 newPos = _target.transform.position;

			// offset z
			newPos.z = -1f;

			// clamp to bounds
			if (OrthographicBounds (Camera.main, newPos).max.x > _currentRoomSprite.bounds.max.x)
			{
				newPos.x = _oldPos.x;
			}
			else
			if (OrthographicBounds (Camera.main, newPos).min.x < _currentRoomSprite.bounds.min.x)
			{
				newPos.x = _oldPos.x;
			}
		
			
			newPos.y = currentRoom.transform.position.y;
			transform.position = newPos;
		}
	}

	public static Bounds OrthographicBounds(Camera camera, Vector3 pos)
	{
		float screenAspect = (float)Screen.width / (float)Screen.height;
		float cameraHeight = camera.orthographicSize * 2;
		Bounds bounds = new Bounds(
			pos,
			new Vector3(cameraHeight * screenAspect, cameraHeight, 0));
		return bounds;
	}

	public void MoveRooms(Room room)
	{
		currentRoom = room;
		_currentRoomSprite = currentRoom.backgroundSprite;
		_transitioning = true;

		StartCoroutine (Readjust ());
	}

	IEnumerator Readjust()
	{
		yield return new WaitForSeconds (0.5f);

		// save pos
		Vector3 pos = transform.position;

		// adjust x
		if (_target.position.x > currentRoom.transform.position.x)
		{
			pos.x = currentRoom.backgroundSprite.bounds.max.x - OrthographicBounds (Camera.main, transform.position).size.x * 0.5f;
		}
		else
		{
			pos.x = currentRoom.backgroundSprite.bounds.min.x + OrthographicBounds (Camera.main, transform.position).size.x * 0.5f;
		}

		// apply!
		transform.position = pos;
	}

}
