﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Linq;
using System.Collections.Generic;

public class EndDayStats : MonoBehaviour 
{
	public Image sanityBarFill;
	public Text sanityBarCounter;

	public Image resourcesBarFill;
	public Text resourcesBarCounter;

	public Text resourceUpdater;
	public Text sanityUpdater;

	public Text roofBlockageIntegrityCounter;
	public Text roofBlockageIntegrityUpdater;

	public Text spaceToContinue;

	float sanityWorsening = 0f;

	bool waitForContinue = false;
	bool continueGame = false;

	float amountToReduceRoofBlockage = 5f;

	void OnEnable()
	{
		resourceUpdater.text = "";
		sanityUpdater.text = "";
		roofBlockageIntegrityUpdater.text = "";

		sanityBarFill.fillAmount = Game.Instance._sanityBarImage.fillAmount;
		sanityBarCounter.text = Game.Instance._sanityCounter.text;

		resourcesBarFill.fillAmount = Game.Instance._resourceBarImage.fillAmount;
		resourcesBarCounter.text = Game.Instance._resourceCounter.text;

		roofBlockageIntegrityCounter.text = Game.Instance.roofBlockageAmount.text;

		GetComponent<CanvasGroupFader> ().FadeIn (0.5f);
		StartCoroutine (ShowEndLevelStats ());

		sanityWorsening += 0.1f;
	}

	IEnumerator ShowEndLevelStats()
	{
		yield return new WaitForSeconds (2f);

		resourceUpdater.GetComponent<CanvasGroupFader> ().FadeIn (0.1f);

		// do resource updates

		// FUSE BOX
		ActionTrigger fuseBox = Game.Instance.actionTriggers.Find(x => x.itemName == "Fuse Box");
		resourceUpdater.text = fuseBox.broken ? "Fuse Box Broken (Irrepairable)" : "Fuse Box Working (+5 Resource)";

		if(!fuseBox.broken)
		{
			Game.Instance.currentNumResources += 5f;
		}

		// work out amount of resources available
		Game.Instance._resourceBarImage.fillAmount = (1 / Game.Instance.maxResources) * Game.Instance._currentNumResources;
		Game.Instance._resourceCounter.text = Game.Instance._currentNumResources.ToString () + "/" + Game.Instance.maxResources.ToString ();

		resourcesBarFill.fillAmount = Game.Instance._resourceBarImage.fillAmount;
		resourcesBarCounter.text = Game.Instance._resourceCounter.text;

		yield return new WaitForSeconds (1f);

		// OVEN
		ActionTrigger oven = Game.Instance.actionTriggers.Find(x => x.itemName == "Oven");
		resourceUpdater.text = oven.broken ? "Oven Broken (Irrepairable)" : "Oven Working (+5 Resource)";

		if(!oven.broken)
		{
			Game.Instance.currentNumResources += 5f;
		}

		// work out amount of resources available
		Game.Instance._resourceBarImage.fillAmount = (1 / Game.Instance.maxResources) * Game.Instance._currentNumResources;
		Game.Instance._resourceCounter.text = Game.Instance._currentNumResources.ToString () + "/" + Game.Instance.maxResources.ToString ();

		resourcesBarFill.fillAmount = Game.Instance._resourceBarImage.fillAmount;
		resourcesBarCounter.text = Game.Instance._resourceCounter.text;

		yield return new WaitForSeconds (1f);

		// OVEN
		ActionTrigger fridge = Game.Instance.actionTriggers.Find(x => x.itemName == "Refrigerator");
		resourceUpdater.text = fridge.broken ? "Refrigerator Broken (Irrepairable)" : "Refrigerator Working (+5 Resource)";

		if(!fridge.broken)
		{
			Game.Instance.currentNumResources += 5f;
		}

		// work out amount of resources available
		Game.Instance._resourceBarImage.fillAmount = (1 / Game.Instance.maxResources) * Game.Instance._currentNumResources;
		Game.Instance._resourceCounter.text = Game.Instance._currentNumResources.ToString () + "/" + Game.Instance.maxResources.ToString ();

		resourcesBarFill.fillAmount = Game.Instance._resourceBarImage.fillAmount;
		resourcesBarCounter.text = Game.Instance._resourceCounter.text;

		yield return new WaitForSeconds (1f);

		resourceUpdater.GetComponent<CanvasGroupFader> ().FadeOut (0.1f);
		sanityUpdater.GetComponent<CanvasGroupFader> ().FadeIn (0.1f);

		// Sanity
		sanityUpdater.text = "Spirit Attacks Worsen";
		Game.Instance.currentSanity -= (2f + sanityWorsening);

		sanityBarFill.fillAmount = Game.Instance._sanityBarImage.fillAmount;
		sanityBarCounter.text = Game.Instance._sanityCounter.text;

		yield return new WaitForSeconds (1f);

		if (!Game.Instance.humanContactForTheDay)
		{
			// Sanity
			sanityUpdater.text = "No Human Contact Today";
			Game.Instance.currentSanity -= (2f + sanityWorsening);

			sanityBarFill.fillAmount = Game.Instance._sanityBarImage.fillAmount;
			sanityBarCounter.text = Game.Instance._sanityCounter.text;

			yield return new WaitForSeconds (1f);
		}

		int numBrokeThroughBarriers = 0;
		foreach(ActionTrigger at in Game.Instance.actionTriggers)
		{
			if (at.actionType == ActionTrigger.ACTION_TYPE.STRENGTHEN)
			{
				if (at._currentStrengthenTier == ActionTrigger.STRENGTHEN_TIERS.BROKEN)
				{
					numBrokeThroughBarriers++;
				}
			}
		}

		if (numBrokeThroughBarriers > 0)
		{
			sanityUpdater.text = numBrokeThroughBarriers.ToString () + " Got through defences";

			Game.Instance.currentSanity -= (1f * numBrokeThroughBarriers);

			sanityBarFill.fillAmount = Game.Instance._sanityBarImage.fillAmount;
			sanityBarCounter.text = Game.Instance._sanityCounter.text;
		}

		yield return new WaitForSeconds (1f);

		int overunHours = (Game.Instance.hoursUsed - Game.Instance.hoursUsedAim);
		if (overunHours > 0)
		{
			sanityUpdater.text = "Lost " + overunHours.ToString() + " Hours Sleep";
			Game.Instance.currentSanity -= (0.5f * overunHours);
		}
		else
		{
			sanityUpdater.text = "No loss of sleep";
		}

		Game.Instance.hoursUsed = 0;

		sanityBarFill.fillAmount = Game.Instance._sanityBarImage.fillAmount;
		sanityBarCounter.text = Game.Instance._sanityCounter.text;

		int countDead = 0;
		int countAlive = 0;
		int numAliveSurvivors = 0;
		List<ActionTrigger> deadFamilies = new List<ActionTrigger> ();
		List<ActionTrigger> aliveFamilies = new List<ActionTrigger> ();

		foreach(ActionTrigger at in Game.Instance.actionTriggers)
		{
			if (at.familyDiscovered && at.actionType == ActionTrigger.ACTION_TYPE.FAMILY && at._currentFamilyTier == ActionTrigger.FAMILY_TIERS.VULNERABLE)
			{
				deadFamilies.Add (at);
			}
	
			if (at.familyDiscovered && at.actionType == ActionTrigger.ACTION_TYPE.FAMILY && at._currentFamilyTier == ActionTrigger.FAMILY_TIERS.PROTECTED)
			{
				aliveFamilies.Add (at);
			}
		}

		while (countDead < deadFamilies.Count)
		{
			yield return new WaitForSeconds (1f);

			sanityUpdater.text = deadFamilies [countDead].itemName + " Died!";

			Game.Instance.currentSanity -= 5f;
			sanityBarFill.fillAmount = Game.Instance._sanityBarImage.fillAmount;
			sanityBarCounter.text = Game.Instance._sanityCounter.text;

			countDead++;
		}

		yield return new WaitForSeconds (1f);

		while (countAlive < aliveFamilies.Count)
		{
			yield return new WaitForSeconds (1f);

			sanityUpdater.text = aliveFamilies [countAlive].itemName + " Survived the night!";

			Game.Instance.currentSanity += 2;
			sanityBarFill.fillAmount = Game.Instance._sanityBarImage.fillAmount;
			sanityBarCounter.text = Game.Instance._sanityCounter.text;

			countAlive++;
		}

		yield return new WaitForSeconds (1f);
		sanityUpdater.GetComponent<CanvasGroupFader> ().FadeOut (0.1f);
		roofBlockageIntegrityUpdater.GetComponent<CanvasGroupFader> ().FadeIn (0.1f);

		roofBlockageIntegrityUpdater.text = countAlive.ToString () + " Survivors to Help";

		numAliveSurvivors = aliveFamilies.Count;

		float rbp = Game.Instance._roofBlockagePercentage;
		rbp -= numAliveSurvivors > 0 ? amountToReduceRoofBlockage * (1 + numAliveSurvivors) : amountToReduceRoofBlockage;
		Game.Instance.roofBlockagePercentage = rbp;

		roofBlockageIntegrityCounter.text = Game.Instance.roofBlockageAmount.text;

		yield return new WaitForSeconds (1f);

		roofBlockageIntegrityUpdater.GetComponent<CanvasGroupFader> ().FadeOut (0.1f);

		bool win = false;
		if (Game.Instance._roofBlockagePercentage <= 0)
		{
			spaceToContinue.text = "You made it out alive, press space to restart";
			win = true;
		}

		bool insane = false;
		if(Game.Instance.currentSanity <= 0)
		{
			spaceToContinue.text = "You have gone insane, press space to return to title";
			insane = true;
			win = false;
		}
		spaceToContinue.GetComponent<CanvasGroupFader> ().FadeIn(0.5f);

		foreach (ActionTrigger at in Game.Instance.actionTriggers)
		{
			if (at.actionType == ActionTrigger.ACTION_TYPE.FAMILY)
			{
				if (at.familyDiscovered)
				{
					at.ApplyDetriment (0);
				}
			}
		}

		Game.Instance.humanContactForTheDay = false;


		yield return new WaitForSeconds (0.6f);

		waitForContinue = true;

		while(!continueGame)
		{
			yield return null;
		}

		GetComponent<CanvasGroupFader> ().FadeOut (0.5f);
		spaceToContinue.GetComponent<CanvasGroupFader> ().FadeOut(0.5f);

		yield return new WaitForSeconds (0.5f);

		if (insane)
		{
			Application.LoadLevel ("Ritual Main Scene");
		}
		else
		{
			if (win)
			{
				Application.LoadLevel ("Ritual Main Scene");
			}
			else
			{
				waitForContinue = false;
				continueGame = false;
				this.enabled = false;
			}
		}
	}

	void Update()
	{
		if (!waitForContinue)
			return;

		if (Input.GetKeyDown (KeyCode.Space))
		{
			continueGame = true;
		}

	}
}
