﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public class MapControl : MonoBehaviour 
{
	Apartment[] apartments;
	public Apartment startApartment;

	Apartment currentSelectedApartment;

	CanvasGroupFader fadeToBlack;
	CanvasGroupFader fader;

	public ActionTrigger currentActionTrigger;

	float t = 0f;
	float cooldownTime = 0.2f;
	bool _coolingDown = false;

	[System.Serializable]
	public class ApartmentLookupValue
	{
		public string physicalApartmentName;
		public Apartment mapApartment;

	}
	public List<ApartmentLookupValue> apartmentLookups = new List<ApartmentLookupValue>();


	void Awake()
	{
		apartments = Resources.FindObjectsOfTypeAll<Apartment> ();
		currentSelectedApartment = startApartment;
		currentSelectedApartment.selected = true;

		fadeToBlack = GameObject.Find ("FadeToBlack").GetComponent<CanvasGroupFader> ();
		fader = GetComponent<CanvasGroupFader> ();
	}

	void OnEnable()
	{
		currentSelectedApartment = apartmentLookups.Find (x => x.physicalApartmentName == Player.Instance.currentApartment.gameObject.name).mapApartment;
		currentSelectedApartment.selected = true;
		if (currentSelectedApartment.name == Player.Instance.currentApartment.gameObject.name)
		{
			currentSelectedApartment.UpdateDetailStay ();
		}
		fader.FadeIn (0.5f);

	}

	void OnDisable()
	{
		fader.FadeOut (0.5f);
	}

	void Update()
	{
		if (currentSelectedApartment.name == Player.Instance.currentApartment.gameObject.name)
		{
			currentSelectedApartment.UpdateDetailStay ();
		}
		else
		{
			currentSelectedApartment.UpdateDetailMove ();
		}

		if (Input.GetKeyDown (KeyCode.DownArrow) && !_coolingDown)
		{
			_coolingDown = true;
			Selectable selected = currentSelectedApartment.GetComponent<Button> ().navigation.selectOnDown;
			if (selected != null)
			{
				currentSelectedApartment.selected = false;
				currentSelectedApartment = selected.GetComponent<Apartment> ();
				currentSelectedApartment.selected = true;
			}
			return;
		}

		if (Input.GetKeyDown (KeyCode.UpArrow) && !_coolingDown)
		{
			_coolingDown = true;
			Selectable selected = currentSelectedApartment.GetComponent<Button> ().navigation.selectOnUp;
			if (selected != null)
			{
				currentSelectedApartment.selected = false;
				currentSelectedApartment = selected.GetComponent<Apartment> ();
				currentSelectedApartment.selected = true;
			}
			return;
		}

		if (Input.GetKeyDown (KeyCode.RightArrow) && !_coolingDown)
		{
			_coolingDown = true;
			Selectable selected = currentSelectedApartment.GetComponent<Button> ().navigation.selectOnRight;
			if (selected != null)
			{
				currentSelectedApartment.selected = false;
				currentSelectedApartment = selected.GetComponent<Apartment> ();
				currentSelectedApartment.selected = true;
			}
			return;
		}

		if (Input.GetKeyDown (KeyCode.LeftArrow) && !_coolingDown)
		{
			_coolingDown = true;
			Selectable selected = currentSelectedApartment.GetComponent<Button> ().navigation.selectOnLeft;
			if (selected != null)
			{
				currentSelectedApartment.selected = false;
				currentSelectedApartment = selected.GetComponent<Apartment> ();
				currentSelectedApartment.selected = true;
			}
			return;
		}

		if (Input.GetKeyDown (KeyCode.Space) && !_coolingDown)
		{
			fadeToBlack.GetComponentInChildren<Text> ().text = "";
			fadeToBlack.FadeIn (0.1f);
			_coolingDown = true;
			if (currentSelectedApartment.name != Player.Instance.currentApartment.gameObject.name)
			{
				Game.Instance.SpendTime (currentActionTrigger);
			}
			PhysicalApartment pa = GameObject.Find ("Apartments/" + currentSelectedApartment.name).GetComponent<PhysicalApartment> ();
			Player.Instance.TravelTo (pa);
			this.enabled = false;
		}


		if (_coolingDown)
		{
			t += Time.deltaTime;
			if (t >= cooldownTime)
			{
				t = 0f;
				_coolingDown = false;
			}
		}
	}
}
