﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class CanvasGroupFader : MonoBehaviour 
{
	public CanvasGroup fadeableCanvasGroup;
	public bool fadeInOnStart = false;
	public bool fadeOutOnStart = false;
	public float fadeDelay = 0f;
	public float fadeTime = 0.5f;

	private bool logInitialFadeSequence = false;

	public bool startFaded = false;
	public float savedAlpha = 0f;

	void Awake()
	{
		savedAlpha = fadeableCanvasGroup.alpha;
		fadeableCanvasGroup.alpha = startFaded ? 0f : savedAlpha == 0 ? 1f : savedAlpha;
	}

	IEnumerator Start()
	{
		yield return new WaitForSeconds(fadeDelay);

		if(fadeInOnStart)
		{
			FadeIn(fadeTime);
		}
		else if(fadeOutOnStart)
		{
			FadeOut(fadeTime);
		}
	}

	// check the alpha value of most opaque object
	float MaxAlpha()
	{
		return savedAlpha;
	}

	public void FadeIn()
	{
		FadeIn (fadeTime);
	}

	public void FadeOut()
	{
		FadeOut (fadeTime);
	}

	public void FadeIn(float newFadeTime, float delay = 0)
	{
		StopAllCoroutines ();
		StartCoroutine (DoFade(newFadeTime, delay));
	}
	
	public void FadeOut(float newFadeTime, float delay = 0)
	{
		StopAllCoroutines ();
		StartCoroutine (DoFade (-newFadeTime, delay));
	}

	IEnumerator DoFade(float fadingTime, float delay = 0)
	{
		yield return new WaitForSeconds (delay);

		// log fading direction, then precalculate fading speed as a multiplier
		bool fadingOut = (fadingTime < 0.0f);
		float fadingOutSpeed = 1.0f / fadingTime; 

		float alphaValue = fadeableCanvasGroup.alpha;

		if (logInitialFadeSequence && !fadingOut)
		{
			alphaValue = 0f; 
			logInitialFadeSequence = false; 
		}

		// iterate to change alpha value 
		while ( (alphaValue >= 0.0f && fadingOut) || (alphaValue <= 1.0f && !fadingOut)) 
		{
			alphaValue += Time.deltaTime * fadingOutSpeed; 

			fadeableCanvasGroup.alpha = alphaValue;
			
			yield return null; 
		}
	}
}
