﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;
using System.Collections.Generic;
using System.Linq;

public class Game : Singleton<Game> 
{
	protected Game() {}
	CanvasGroupFader _fadeToBlack;

	public bool humanContactForTheDay = false;

	public EndDayStats endDayStats;

	public float _roofBlockagePercentage = 100f;
	public float roofBlockagePercentage
	{
		set
		{
			_roofBlockagePercentage = value;
			roofBlockageAmount.text = _roofBlockagePercentage.ToString () + "%";
		}
	}

	public int numDailyTimewastersForSanity = 4;
	bool doneTimewasting = false;
	bool _advancingDay = false;

	public Text roofBlockageAmount;

	public int numAliveSurvivors = 0;

	List<ActionTrigger> currentTimewasters = new List<ActionTrigger> ();

	CanvasGroupFader pressSpaceToContinue;

	Apartment currentApartment;


	public enum PICKUPS
	{
		NONE = 0,
		FRONT_DOOR_KEY = 1
	}

	CanvasGroupFader _gameText;
	CanvasGroupFader _itemNameText;
	Text _gameTextTxt;
	Text _itemNameTextTxt;

	public GameObject checklistTrigger;
	public GameObject keyTrigger;

	public string firstDayText;
	public string nextDayText;

	string fatiqueText = "Fatique kicks in, you head to bed";

	bool _startedMonologue = false;
	bool _allowSkip = false;
	bool _dayStarted = false;
	bool _skipped = false;

	public bool hasKey = false;

	Player _player;

	public string[] innerMonologueStartDays;

	public int maxHoursInDay = 20;
	public int hoursUsedAim = 16;
	public int hoursUsed = 0;
	public int currentTime = 8;

	public bool isDayOne = true;
	public int currentDay = 0;
	string ampm = "am";

	public float maxResources = 15;
	public float _currentNumResources = 15f;
	public float maxSanity = 15f;
	public float _currentSanity = 15f;

	public Image _resourceBarImage;
	public Text _resourceCounter;
	Text _clock;

	public Image _sanityBarImage;
	public Text _sanityCounter;

	public List<ActionTrigger> actionTriggers = new List<ActionTrigger>();

	void Awake()
	{
		_fadeToBlack = GameObject.Find ("FadeToBlack").GetComponent<CanvasGroupFader> ();
		_fadeToBlack.GetComponentInChildren<Text> ().text = firstDayText;

		_gameText = GameObject.Find ("GameText").GetComponent<CanvasGroupFader> ();
		_gameTextTxt = _gameText.GetComponent<Text> ();

		_itemNameText = GameObject.Find ("ItemNameText").GetComponent<CanvasGroupFader> ();
		_itemNameTextTxt = _itemNameText.GetComponent<Text> ();

		_player = GameObject.Find ("Player").GetComponent<Player> ();

		ActionTrigger[] temp = Resources.FindObjectsOfTypeAll<ActionTrigger> ();
		actionTriggers = temp.ToList ();

		pressSpaceToContinue = GameObject.Find ("PressSpace").GetComponent<CanvasGroupFader> ();


		// RESOURCE BAR
		_resourceBarImage = GameObject.Find("ResourceBar/BarBG/BarFill").GetComponent<Image>();
		_resourceCounter = GameObject.Find("ResourceBar/BarBG/Counter").GetComponent<Text>();

		_resourceBarImage.fillAmount = (1 / maxResources) * _currentNumResources;
		_resourceCounter.text = _currentNumResources.ToString () + "/" + maxResources.ToString ();

		// Clock
		_clock = GameObject.Find("Clock").GetComponent<Text>();
		_clock.text = currentTime.ToString() + ":" + "00 " + ampm;

		// SANITY BAR
		_sanityBarImage = GameObject.Find("SanityBar/BarBG/BarFill").GetComponent<Image>();
		_sanityCounter = GameObject.Find("SanityBar/BarBG/Counter").GetComponent<Text>();

		_sanityBarImage.fillAmount = (1 / maxSanity) * currentSanity;
		_sanityCounter.text = currentSanity.ToString () + "/" + maxSanity.ToString ();

		roofBlockagePercentage = 100f;
	}

	void Update()
	{
		if (_dayStarted)
		{
			// do daily stuff here
			if (currentTimewasters.Count == numDailyTimewastersForSanity && !doneTimewasting)
			{
				doneTimewasting = true;
				currentSanity++;
			}
			return;
		}


		if (_allowSkip)
		{
			if (Input.GetKeyDown (KeyCode.Space))
			{
				_skipped = true;
				pressSpaceToContinue.FadeOut (0.1f);
			}
		}
		
		if (_fadeToBlack.fadeableCanvasGroup.alpha == 0 && !_startedMonologue)
		{
			StartCoroutine(StartMonologue ());
		}
	}

	IEnumerator StartMonologue()
	{
		_startedMonologue = true;

		_gameTextTxt.text = innerMonologueStartDays [currentDay > 3 ? 3 : currentDay];
		_gameText.FadeIn (0.5f);

		yield return new WaitForSeconds (2f);

		_allowSkip = true;
		pressSpaceToContinue.FadeIn (0.1f);

		while (!_skipped)
		{
			yield return null;
		}

		_gameText.FadeOut (0.5f);

		yield return new WaitForSeconds (0.6f);

		_player.transitioning = false;
		_dayStarted = true;
		_skipped = false;
		_startedMonologue = false;
		_allowSkip = false;
	}

	public float currentNumResources
	{
		get 
		{
			return _currentNumResources;
		}

		set
		{
			if (_currentNumResources != value)
			{
				_currentNumResources = value;
				_currentNumResources = Mathf.Clamp (_currentNumResources, 0, 15f);
				_resourceBarImage.fillAmount = (1 / maxResources) * _currentNumResources;
				_resourceCounter.text = _currentNumResources.ToString () + "/" + maxResources.ToString ();
			}	
		}
	}

	public float currentSanity
	{
		get 
		{
			return _currentSanity;
		}

		set
		{
			if (_currentSanity != value)
			{
				_currentSanity = value;
				_currentSanity = Mathf.Clamp (_currentSanity, 0, 15f);
				_sanityBarImage.fillAmount = (1 / maxSanity) * _currentSanity;
				_sanityCounter.text = _currentSanity.ToString () + "/" + maxSanity.ToString ();
			}	
		}
	}

	public void SpendResource(ActionTrigger at, int numResources = 1)
	{
		if ((_currentNumResources - numResources) >= 0)
		{
			currentNumResources-= numResources;
		}
	}

	public void SpendTime(ActionTrigger at, int numHours = 1)
	{
		hoursUsed += numHours;
		currentTime += numHours;
		if (currentTime > 12)
		{
			currentTime = 1;
			if (ampm == "am")
			{
				ampm = "pm";
			}
			else
			{
				ampm = "am";
			}
		}

		_clock.text = currentTime.ToString() + ":" + "00 " + ampm;
		if (hoursUsed >= maxHoursInDay && !_advancingDay)
		{
			at.ResetStuffForAdvance ();
			_fadeToBlack.GetComponentInChildren<Text> ().text = fatiqueText;
			AdvanceDay ();
		}
	}

	public void UpdateAndShowItemName(string itemname)
	{
		_itemNameTextTxt.text = itemname;
		_itemNameText.FadeIn (0.2f);
	}

	public void HideItemName()
	{
		_itemNameText.FadeOut (0.2f);
	}

	public void AdvanceDay()
	{ 
		_advancingDay = true;
		Player.Instance.transitioning = true;
		// fade to blackTime
		_fadeToBlack.FadeIn (1f);

		StartCoroutine (MoveToNextDay ());
	}

	IEnumerator MoveToNextDay()
	{
		yield return new WaitForSeconds (1.1f);

		Player.Instance.MoveRooms (GameObject.Find ("Bedroom").GetComponent<Room> (), GameObject.Find ("MainStartPoint").transform, false, true);

		_gameTextTxt.text = "";
		_gameText.FadeOut (0f);

		_dayStarted = false;

		currentTimewasters = new List<ActionTrigger> ();
		doneTimewasting = false;

		currentDay++;

		if (currentDay == 1)
		{
			_fadeToBlack.GetComponentInChildren<Text> ().text = nextDayText;
			checklistTrigger.SetActive (true);
			currentSanity -= 5;
			isDayOne = false;
		}

		if (currentDay == 2)
		{
			keyTrigger.SetActive (true);
		}

		foreach (ActionTrigger at in actionTriggers)
		{
			if (at.actionType != ActionTrigger.ACTION_TYPE.FAMILY)
			{
				at.ApplyDetriment (0);
			}
		}

		// 8 oclock again
		currentTime = 8;

		ampm = "am";
		_clock.text = currentTime.ToString() + ":" + "00 " + ampm;

		// day 2 has forced amount of detriment
		if (currentDay == 1)
		{
			//no hours used
			hoursUsed = 0;
			foreach (ActionTrigger at in actionTriggers)
			{
				if (at.actionType == ActionTrigger.ACTION_TYPE.MAINTENANCE && !at.broken)
				{
					_currentNumResources += 5f;
					_currentNumResources = Mathf.Clamp (_currentNumResources, 0, 15f);
				}
			}

			// work out amount of resources available
			_resourceBarImage.fillAmount = (1 / maxResources) * _currentNumResources;
			_resourceCounter.text = _currentNumResources.ToString () + "/" + maxResources.ToString ();
		}

		yield return new WaitForSeconds (2f);

		 //fade to black
		if (currentDay == 1)
		{
			_fadeToBlack.FadeOut (1f);
			_advancingDay = false;
		}
		else
		{
			endDayStats.enabled = true;
			while(endDayStats.enabled)
			{
				yield return null;
			}
			_fadeToBlack.FadeOut (1f);
			_advancingDay = false;
		}
	}

	public void Pickup(PICKUPS pickup)
	{
		switch (pickup)
		{
		case PICKUPS.NONE:
			// nowt
			break;
		case PICKUPS.FRONT_DOOR_KEY:
			hasKey = true;
			break;
		}
	}

	public void AddToTimewasters(ActionTrigger at)
	{
		if (!currentTimewasters.Contains (at))
		{
			currentTimewasters.Add (at);
		}
	}
}
