﻿using UnityEngine;
using System.Collections;

public class Player : Singleton<Player> 
{
	Animator _animator;
	SpriteRenderer _spriteRenderer;
	Vector3 _oldPos;
	CameraScript _cameraScript;

	public Room currentRoom;
	public float movementSpeed;
	CanvasGroupFader _fadeToBlack;

	Collider2D _collider;

	Transform _mainStartPoint;

	public PhysicalApartment currentApartment;

	bool _interacting = false;
	public bool interacting
	{
		set
		{
			_interacting = value;
		}
	}

	bool _transitioning = true;
	public bool transitioning
	{
		set
		{
			_transitioning = value;
		}
	}


	void Awake()
	{
		_animator = GetComponentInChildren<Animator> ();
		_spriteRenderer = GetComponentInChildren<SpriteRenderer> ();

		_mainStartPoint = GameObject.Find ("MainStartPoint").transform;
		transform.position = _mainStartPoint.position;

		_cameraScript = Camera.main.GetComponent<CameraScript> ();
		_fadeToBlack = GameObject.Find ("FadeToBlack").GetComponent<CanvasGroupFader> ();

		_collider = GetComponentInChildren<Collider2D> ();
	}

	void Update()
	{
		_collider.enabled = !_transitioning && !_interacting;
		if (_interacting)
		{
			_animator.CrossFade ("Interacting", 0f);
			return;
		}

		if (_transitioning)
		{
			_animator.CrossFade ("Idle", 0f);
			return;
		}
			
		_oldPos = transform.position;
		Vector3 newPos = transform.position;

		// MOVING
		if (Input.GetKey (KeyCode.RightArrow))
		{
			
			newPos.x += movementSpeed * Time.deltaTime;

			// animate
			_animator.CrossFade ("Walking", 0f);

			// flip
			_spriteRenderer.flipX = false;
		}
		else if (Input.GetKey (KeyCode.LeftArrow))
		{
			newPos.x -= movementSpeed * Time.deltaTime;

			// animate
			_animator.CrossFade ("Walking", 0f);

			// flip
			_spriteRenderer.flipX = true;
		}


		// IDLE
		if (newPos.x == transform.position.x)
		{
			_animator.CrossFade ("Idle", 0f);
		}

		if (!currentRoom.validLeft && newPos.x - (_spriteRenderer.bounds.size.x * 0.5f) < Camera.main.ViewportToWorldPoint(Vector3.zero).x)
		{
			newPos.x = _oldPos.x;
		}

		if (!currentRoom.validRight && newPos.x + (_spriteRenderer.bounds.size.x * 0.5f) > Camera.main.ViewportToWorldPoint(Vector3.one).x)
		{
			newPos.x = _oldPos.x;
		}

		// update position
		transform.position = newPos;
	}

	public void MoveRooms(Room room, Transform startPoint, bool allowToReenableControl = true, bool resetFlip = false)
	{
		_transitioning = true;
		transform.position = startPoint.position;
		currentRoom = room;
		_cameraScript.MoveRooms (room);

		if (resetFlip)
		{
			_spriteRenderer.flipX = false;
		}

		if(allowToReenableControl)
		StartCoroutine (EnableControl ());
	}

	IEnumerator EnableControl()
	{
		yield return new WaitForSeconds (0.5f);

		if (_fadeToBlack.fadeableCanvasGroup.alpha == 1)
		{
			_fadeToBlack.FadeOut (1f);
		}

		while (_fadeToBlack.fadeableCanvasGroup.alpha > 0)
		{
			yield return null;
		}

		yield return new WaitForSeconds (0.5f);

		_transitioning = false;
		_cameraScript.transitioning = false;
	}

	public void TravelTo(PhysicalApartment pa)
	{
		currentApartment = pa;
		MoveRooms (pa.startRoom, pa.mainStartPoint, false);

		StartCoroutine (MovedApartments ());
	}

	IEnumerator MovedApartments()
	{
		yield return new WaitForSeconds (0.5f);

		_cameraScript.transitioning = false;
		_fadeToBlack.FadeOut (0.5f);

		yield return new WaitForSeconds (0.5f);

		transitioning = false;
	}
}
